@@ -15,9 +15,8 @@ use HTML::Display qw();
 use HTML::TokeParser::Simple;
 use B::Deparse;
 
-use vars qw( $VERSION @EXPORT %munge_map );
-$VERSION = '0.54';
-@EXPORT = qw( &shell );
+our $VERSION = '0.58';
+our @EXPORT = qw( &shell );
 
 =head1 NAME
 
@@ -86,13 +85,35 @@ filename for the rcfile, the C<rcfile> parameter can be passed to the constructo
 
   rcfile => '.myapprc',
 
+=over 4
+
+=item B<agent>
+
+  my $shell = WWW::Mechanize::Shell->new(
+      agent => WWW::Mechanize::Chrome->new(),
+  );
+
+Pass in a premade custom user agent. This object must be compatible to
+L<WWW::Mechanize>. Use this feature from the command line as
+
+  perl -Ilib -MWWW::Mechanize::Chrome \
+             -MWWW::Mechanize::Shell \
+             -e"shell(agent => WWW::Mechanize::Chrome->new())"
+
+=back
+
 =cut
 
 sub init {
   my ($self) = @_;
   my ($name,%args) = @{$self->{API}{args}};
 
-  $self->{agent} = WWW::Mechanize->new();
+  $self->{agent} = $args{ agent };
+  if( ! $self->agent ) {
+      my $class = $args{ agent_class } || 'WWW::Mechanize';
+      my $args  = $args{ agent_args }  || [];
+      $self->{agent} = $class->new( @$args );
+  };
 
   $self->{formfiller} = WWW::Mechanize::FormFiller->new(default => [ Ask => $self ]);
 
@@ -119,12 +140,13 @@ sub init {
   $self->{redirect_ok_wrapper} = wrap 'WWW::Mechanize::redirect_ok',
     post => sub {
         return unless $_[1];
-        $self->status( "\nRedirecting to ".$_[1]->uri."\n" ); 
-        $_[-1] 
+        $self->status( "\nRedirecting to ".$_[1]->uri."\n" );
+        $_[-1]
     };
 
   # Load the proxy settings from the environment
-  $self->agent->env_proxy();
+  $self->agent->env_proxy()
+      if $self->agent->can('env_proxy');
 
   # Read our .rc file :
   # I could use File::Homedir, but the docs claim it dosen't work on Win32. Maybe
@@ -292,7 +314,7 @@ sub sync_browser {
   return unless $self->agent->res;
 
   # Prepare the HTML for local display :
-  my $unclean = $self->agent->res->content;
+  my $unclean = $self->agent->content;
   my $html = '';
 
   # ugly fix:
@@ -590,12 +612,12 @@ sub run_save {
   if (ref $user_link) {
     my $count = -1;
     my $re = $user_link;
-    @links = map { $count++; (($_->text =~ /$re/)||($_->url =~ /$re/)) ? $count : () } @all_links;
+    @links = map { $count++; ((defined $_->text && $_->text =~ /$re/)||(defined $_->url && $_->url =~ /$re/)) ? $count : () } @all_links;
     if (@links == 0) {
       print "No match for /$re/.\n";
     };
     push @history, q{my $count = -1;} . "\n";
-    push @history, sprintf q{@links = map { $count++; (($_->text =~ qr(%s))||($_->url =~ qr(%s))) ? $count : () } @all_links;} . "\n", $re, $re;
+    push @history, sprintf q{@links = map { $count++; ((defined $_->text && $_->text =~ qr(%s))||(defined $_->url && $_->url =~ qr(%s))) ? $count : () } @all_links;} . "\n", $re, $re;
   } else {
     @links = $user_link;
     push @history, sprintf q{@links = '%s';} . "\n", $user_link;
@@ -685,7 +707,7 @@ sub run_title {
 
 Prints all C<< <H1> >> through C<< <H5> >> strings found in the content,
 indented accordingly.  With an argument, prints only those
-levels; e.g., C<headers 145> prints H1,H4,H5 strings only. 
+levels; e.g., C<headers 145> prints H1,H4,H5 strings only.
 
 =cut
 
@@ -787,6 +809,23 @@ sub run_links {
   };
 };
 
+=head2 images
+
+Display images on a page
+
+=cut
+
+sub run_images {
+    my ($self) = @_;
+
+    my @images = $self->agent->images;
+    my $count  = 0;
+
+    for my $image ( @images ) {
+        print sprintf("[%d] \"%s\" %s\n", $count++, $image->alt, $image->url);
+    }
+}
+
 =head2 parse
 
 Dump the output of HTML::TokeParser of the current content
@@ -1105,9 +1144,9 @@ sub run_open {
     my $re = $link if ref $link;
     my $count = -1;
     my @possible_links = $self->agent->links();
-    my @links = defined $re 
-        ? map { $count++; $_->text =~ /$re/ ? $count : () } @possible_links
-        : map { $count++; $_->text eq $link ? $count : () } @possible_links;
+    my @links = defined $re
+        ? map { $count++; my $t = $_->text; defined $t && $t =~ /$re/ ? $count : () } @possible_links
+        : map { $count++; my $t = $_->text; defined $t && $t eq $link ? $count : () } @possible_links;
     if (@links > 1) {
       $self->print_pairs([ @links ],[ map {$possible_links[$_]->[1]} @links ]);
       undef $link;
@@ -1324,7 +1363,7 @@ sub run_fillout {
   };
   warn $@ if $@;
   $self->add_history( join( "\n",
-                      map { sprintf( q[$formfiller->add_filler( '%s' => Fixed => '%s' );], @$_ ) } @interactive_values) . '$formfiller->fill_form($agent->current_form);');
+                      map { sprintf( q[$formfiller->add_filler( '%s' => Fixed => '%s' );], $_->[0], defined $_->[1] ? $_->[1] : '' ) } @interactive_values) . '$formfiller->fill_form($agent->current_form);');
 };
 
 =head2 auth
@@ -1410,8 +1449,8 @@ sub run_table {
     };
     $code->();
     my $body = $self->munge_code($code);
-    
-    $self->add_history( 
+
+    $self->add_history(
         "require HTML::TableExtract;\n",
         sprintf( 'my @columns = ( %s );'."\n", join( ",", map( { s/(['\\])/\\$1/g; qq('$_') } @columns ))),
         $body
@@ -1578,8 +1617,8 @@ sub run_eval {
         return
     };
     print join "", @res,"\n";
-    
-    
+
+
     my $script_code = $self->munge_code(qq{print $code, "\\n";});
     #warn "Script: $script_code<<";
     $self->add_history( $script_code );
@@ -1734,7 +1773,7 @@ output independent of WWW::Mechanize::Shell.
 
 =cut
 
-%munge_map = (
+our %munge_map = (
         '^{' => '',
         '}$' => '',
         '\$self->print_paged' => 'print ',
@@ -1745,7 +1784,7 @@ output independent of WWW::Mechanize::Shell.
 sub munge_code {
     my ($self, $code) = @_;
     my $body;
-    
+
     if (ref $code) {
         # Munge code
         my $d = B::Deparse->new('-sC');
@@ -1756,11 +1795,11 @@ sub munge_code {
     } else {
         $body = $code
     }
-    
+
     while (my ($key,$val) = each %munge_map) {
         $body =~ s/$key/$val/gs;
     };
-    
+
     $body
 };
 
@@ -1790,12 +1829,12 @@ sub shell {
 };
 
 {
-  package WWW::Mechanize::FormFiller::Value::Ask;
+  package # hide from CPAN
+      WWW::Mechanize::FormFiller::Value::Ask;
   use WWW::Mechanize::FormFiller;
   use base 'WWW::Mechanize::FormFiller::Value::Callback';
 
-  use vars qw( $VERSION );
-  $VERSION = '0.21';
+  our $VERSION = '0.58';
 
   sub new {
     my ($class,$name,$shell) = @_;
@@ -2039,8 +2078,8 @@ invocation of the shell like with CPAN :
 
 =head1 REPOSITORY
 
-The public repository of this module is 
-L<http://github.com/Corion/WWW-Mechanize-Shell>.
+The public repository of this module is
+L<https://github.com/Corion/WWW-Mechanize-Shell>.
 
 =head1 SUPPORT
 
@@ -2051,7 +2090,7 @@ L<http://perlmonks.org/>.
 
 This library is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
-Copyright (C) 2002,2010 Max Maischein
+Copyright (C) 2002-2017 Max Maischein
 
 =head1 AUTHOR
 
